from enum import Enum


class ListAgentTokensSortsItem(str, Enum):
    CREATEDATASC = "createdAt:asc"
    CREATEDATDESC = "createdAt:desc"
    CREATEDBYIDASC = "createdById:asc"
    CREATEDBYIDDESC = "createdById:desc"
    DESCRIPTIONASC = "description:asc"
    DESCRIPTIONDESC = "description:desc"
    NAMEASC = "name:asc"
    NAMEDESC = "name:desc"
    TOKENSTARTATASC = "tokenStartAt:asc"
    TOKENSTARTATDESC = "tokenStartAt:desc"
    UPDATEDATASC = "updatedAt:asc"
    UPDATEDATDESC = "updatedAt:desc"
    UPDATEDBYIDASC = "updatedById:asc"
    UPDATEDBYIDDESC = "updatedById:desc"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.basic_subject_profile_subject_type import BasicSubjectProfileSubjectType
from ..types import UNSET, Unset

T = TypeVar("T", bound="BasicSubjectProfile")


@_attrs_define
class BasicSubjectProfile:
    """
    Attributes:
        id (str): The subject's ID. Example: clm8qv74h000008mlf08scq7k.
        api_token_name (Union[Unset, str]): The API token's name. Returned only when `SubjectType` is `SERVICEKEY`.
            Example: my-token.
        avatar_url (Union[Unset, str]): The URL for the user's profile image. Returned only when `SubjectType` is
            `USER`. Example: https://avatar.url.
        full_name (Union[Unset, str]): The subject's full name. Returned only when `SubjectType` is `USER`. Example:
            Jane Doe.
        subject_type (Union[Unset, BasicSubjectProfileSubjectType]): The subject type. Example: USER.
        username (Union[Unset, str]): The subject's username. Returned only when `SubjectType` is `USER`. Example:
            user1@company.com.
    """

    id: str
    api_token_name: Union[Unset, str] = UNSET
    avatar_url: Union[Unset, str] = UNSET
    full_name: Union[Unset, str] = UNSET
    subject_type: Union[Unset, BasicSubjectProfileSubjectType] = UNSET
    username: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        api_token_name = self.api_token_name

        avatar_url = self.avatar_url

        full_name = self.full_name

        subject_type: Union[Unset, str] = UNSET
        if not isinstance(self.subject_type, Unset):
            subject_type = self.subject_type.value

        username = self.username

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
            }
        )
        if api_token_name is not UNSET:
            field_dict["apiTokenName"] = api_token_name
        if avatar_url is not UNSET:
            field_dict["avatarUrl"] = avatar_url
        if full_name is not UNSET:
            field_dict["fullName"] = full_name
        if subject_type is not UNSET:
            field_dict["subjectType"] = subject_type
        if username is not UNSET:
            field_dict["username"] = username

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        api_token_name = d.pop("apiTokenName", UNSET)

        avatar_url = d.pop("avatarUrl", UNSET)

        full_name = d.pop("fullName", UNSET)

        _subject_type = d.pop("subjectType", UNSET)
        subject_type: Union[Unset, BasicSubjectProfileSubjectType]
        if isinstance(_subject_type, Unset):
            subject_type = UNSET
        else:
            subject_type = BasicSubjectProfileSubjectType(_subject_type)

        username = d.pop("username", UNSET)

        basic_subject_profile = cls(
            id=id,
            api_token_name=api_token_name,
            avatar_url=avatar_url,
            full_name=full_name,
            subject_type=subject_type,
            username=username,
        )

        basic_subject_profile.additional_properties = d
        return basic_subject_profile

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_custom_role_request_scope_type import CreateCustomRoleRequestScopeType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateCustomRoleRequest")


@_attrs_define
class CreateCustomRoleRequest:
    """
    Attributes:
        name (str): The role's name. Example: Deployment_Viewer.
        permissions (list[str]): The permissions included in the role. Example: ['deployment.get'].
        scope_type (CreateCustomRoleRequestScopeType): The scope of the role. Example: DEPLOYMENT.
        description (Union[Unset, str]): The role's description. Example: Subject can only view deployments..
        restricted_workspace_ids (Union[Unset, list[str]]): The IDs of the Workspaces that the role is restricted to.
            Example: ['cldbvzoi20182g8odxt8ehi5i'].
    """

    name: str
    permissions: list[str]
    scope_type: CreateCustomRoleRequestScopeType
    description: Union[Unset, str] = UNSET
    restricted_workspace_ids: Union[Unset, list[str]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        permissions = self.permissions

        scope_type = self.scope_type.value

        description = self.description

        restricted_workspace_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.restricted_workspace_ids, Unset):
            restricted_workspace_ids = self.restricted_workspace_ids

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "permissions": permissions,
                "scopeType": scope_type,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if restricted_workspace_ids is not UNSET:
            field_dict["restrictedWorkspaceIds"] = restricted_workspace_ids

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        permissions = cast(list[str], d.pop("permissions"))

        scope_type = CreateCustomRoleRequestScopeType(d.pop("scopeType"))

        description = d.pop("description", UNSET)

        restricted_workspace_ids = cast(list[str], d.pop("restrictedWorkspaceIds", UNSET))

        create_custom_role_request = cls(
            name=name,
            permissions=permissions,
            scope_type=scope_type,
            description=description,
            restricted_workspace_ids=restricted_workspace_ids,
        )

        create_custom_role_request.additional_properties = d
        return create_custom_role_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ApiTokenRoleEntityType(str, Enum):
    DEPLOYMENT = "DEPLOYMENT"
    ORGANIZATION = "ORGANIZATION"
    WORKSPACE = "WORKSPACE"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class CreateTeamRequestOrganizationRole(str, Enum):
    ORGANIZATION_BILLING_ADMIN = "ORGANIZATION_BILLING_ADMIN"
    ORGANIZATION_MEMBER = "ORGANIZATION_MEMBER"
    ORGANIZATION_OBSERVE_ADMIN = "ORGANIZATION_OBSERVE_ADMIN"
    ORGANIZATION_OBSERVE_MEMBER = "ORGANIZATION_OBSERVE_MEMBER"
    ORGANIZATION_OWNER = "ORGANIZATION_OWNER"

    def __str__(self) -> str:
        return str(self.value)



"""Contains all the data models used in inputs/outputs"""

from .add_team_members_request import AddTeamMembersRequest
from .allowed_ip_address_range import AllowedIpAddressRange
from .allowed_ip_address_ranges_paginated import AllowedIpAddressRangesPaginated
from .api_token import ApiToken
from .api_token_role import ApiTokenRole
from .api_token_role_entity_type import ApiTokenRoleEntityType
from .api_token_type import ApiTokenType
from .api_tokens_paginated import ApiTokensPaginated
from .basic_subject_profile import BasicSubjectProfile
from .basic_subject_profile_subject_type import BasicSubjectProfileSubjectType
from .create_agent_token_request import CreateAgentTokenRequest
from .create_allowed_ip_address_range_request import CreateAllowedIpAddressRangeRequest
from .create_api_token_request import CreateApiTokenRequest
from .create_api_token_request_type import CreateApiTokenRequestType
from .create_custom_role_request import CreateCustomRoleRequest
from .create_custom_role_request_scope_type import CreateCustomRoleRequestScopeType
from .create_team_request import CreateTeamRequest
from .create_team_request_organization_role import CreateTeamRequestOrganizationRole
from .create_user_invite_request import CreateUserInviteRequest
from .create_user_invite_request_role import CreateUserInviteRequestRole
from .default_role import DefaultRole
from .default_role_scope_type import DefaultRoleScopeType
from .deployment_role import DeploymentRole
from .error import Error
from .invite import Invite
from .list_agent_tokens_sorts_item import ListAgentTokensSortsItem
from .list_allowed_ip_address_ranges_sorts_item import ListAllowedIpAddressRangesSortsItem
from .list_api_tokens_sorts_item import ListApiTokensSortsItem
from .list_permission_groups_scope_type import ListPermissionGroupsScopeType
from .list_role_templates_scope_types_item import ListRoleTemplatesScopeTypesItem
from .list_roles_scope_types_item import ListRolesScopeTypesItem
from .list_roles_sorts_item import ListRolesSortsItem
from .list_team_members_sorts_item import ListTeamMembersSortsItem
from .list_teams_sorts_item import ListTeamsSortsItem
from .list_users_sorts_item import ListUsersSortsItem
from .permission_entry import PermissionEntry
from .permission_group import PermissionGroup
from .role import Role
from .role_scope_type import RoleScopeType
from .role_template import RoleTemplate
from .role_template_scope_type import RoleTemplateScopeType
from .role_with_permission import RoleWithPermission
from .role_with_permission_scope_type import RoleWithPermissionScopeType
from .roles_paginated import RolesPaginated
from .subject_roles import SubjectRoles
from .subject_roles_organization_role import SubjectRolesOrganizationRole
from .team import Team
from .team_member import TeamMember
from .team_members_paginated import TeamMembersPaginated
from .team_organization_role import TeamOrganizationRole
from .teams_paginated import TeamsPaginated
from .update_api_token_request import UpdateApiTokenRequest
from .update_api_token_roles_request import UpdateApiTokenRolesRequest
from .update_custom_role_request import UpdateCustomRoleRequest
from .update_team_request import UpdateTeamRequest
from .update_team_roles_request import UpdateTeamRolesRequest
from .update_team_roles_request_organization_role import UpdateTeamRolesRequestOrganizationRole
from .update_user_roles_request import UpdateUserRolesRequest
from .update_user_roles_request_organization_role import UpdateUserRolesRequestOrganizationRole
from .user import User
from .user_organization_role import UserOrganizationRole
from .user_status import UserStatus
from .users_paginated import UsersPaginated
from .workspace_role import WorkspaceRole
from .workspace_role_role import WorkspaceRoleRole

__all__ = (
    "AddTeamMembersRequest",
    "AllowedIpAddressRange",
    "AllowedIpAddressRangesPaginated",
    "ApiToken",
    "ApiTokenRole",
    "ApiTokenRoleEntityType",
    "ApiTokensPaginated",
    "ApiTokenType",
    "BasicSubjectProfile",
    "BasicSubjectProfileSubjectType",
    "CreateAgentTokenRequest",
    "CreateAllowedIpAddressRangeRequest",
    "CreateApiTokenRequest",
    "CreateApiTokenRequestType",
    "CreateCustomRoleRequest",
    "CreateCustomRoleRequestScopeType",
    "CreateTeamRequest",
    "CreateTeamRequestOrganizationRole",
    "CreateUserInviteRequest",
    "CreateUserInviteRequestRole",
    "DefaultRole",
    "DefaultRoleScopeType",
    "DeploymentRole",
    "Error",
    "Invite",
    "ListAgentTokensSortsItem",
    "ListAllowedIpAddressRangesSortsItem",
    "ListApiTokensSortsItem",
    "ListPermissionGroupsScopeType",
    "ListRolesScopeTypesItem",
    "ListRolesSortsItem",
    "ListRoleTemplatesScopeTypesItem",
    "ListTeamMembersSortsItem",
    "ListTeamsSortsItem",
    "ListUsersSortsItem",
    "PermissionEntry",
    "PermissionGroup",
    "Role",
    "RoleScopeType",
    "RolesPaginated",
    "RoleTemplate",
    "RoleTemplateScopeType",
    "RoleWithPermission",
    "RoleWithPermissionScopeType",
    "SubjectRoles",
    "SubjectRolesOrganizationRole",
    "Team",
    "TeamMember",
    "TeamMembersPaginated",
    "TeamOrganizationRole",
    "TeamsPaginated",
    "UpdateApiTokenRequest",
    "UpdateApiTokenRolesRequest",
    "UpdateCustomRoleRequest",
    "UpdateTeamRequest",
    "UpdateTeamRolesRequest",
    "UpdateTeamRolesRequestOrganizationRole",
    "UpdateUserRolesRequest",
    "UpdateUserRolesRequestOrganizationRole",
    "User",
    "UserOrganizationRole",
    "UsersPaginated",
    "UserStatus",
    "WorkspaceRole",
    "WorkspaceRoleRole",
)



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_user_invite_request_role import CreateUserInviteRequestRole

T = TypeVar("T", bound="CreateUserInviteRequest")


@_attrs_define
class CreateUserInviteRequest:
    """
    Attributes:
        invitee_email (str): The email of the user to invite. Example: user1@company.com.
        role (CreateUserInviteRequestRole): The user's Organization role. Example: ORGANIZATION_MEMBER.
    """

    invitee_email: str
    role: CreateUserInviteRequestRole
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        invitee_email = self.invitee_email

        role = self.role.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "inviteeEmail": invitee_email,
                "role": role,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        invitee_email = d.pop("inviteeEmail")

        role = CreateUserInviteRequestRole(d.pop("role"))

        create_user_invite_request = cls(
            invitee_email=invitee_email,
            role=role,
        )

        create_user_invite_request.additional_properties = d
        return create_user_invite_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error import Error
from ...models.list_teams_sorts_item import ListTeamsSortsItem
from ...models.teams_paginated import TeamsPaginated
from ...types import UNSET, Response, Unset


def _get_kwargs(
    organization_id: str,
    *,
    names: Union[Unset, list[str]] = UNSET,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 20,
    sorts: Union[Unset, list[ListTeamsSortsItem]] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_names: Union[Unset, list[str]] = UNSET
    if not isinstance(names, Unset):
        json_names = names

    params["names"] = json_names

    params["offset"] = offset

    params["limit"] = limit

    json_sorts: Union[Unset, list[str]] = UNSET
    if not isinstance(sorts, Unset):
        json_sorts = []
        for sorts_item_data in sorts:
            sorts_item = sorts_item_data.value
            json_sorts.append(sorts_item)

    params["sorts"] = json_sorts

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/organizations/{organization_id}/teams",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Error, TeamsPaginated]]:
    if response.status_code == 200:
        response_200 = TeamsPaginated.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = Error.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = Error.from_dict(response.json())

        return response_401

    if response.status_code == 403:
        response_403 = Error.from_dict(response.json())

        return response_403

    if response.status_code == 404:
        response_404 = Error.from_dict(response.json())

        return response_404

    if response.status_code == 500:
        response_500 = Error.from_dict(response.json())

        return response_500

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Error, TeamsPaginated]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    organization_id: str,
    *,
    client: AuthenticatedClient,
    names: Union[Unset, list[str]] = UNSET,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 20,
    sorts: Union[Unset, list[ListTeamsSortsItem]] = UNSET,
) -> Response[Union[Error, TeamsPaginated]]:
    """List Teams

     List all Teams in an Organization.

    Args:
        organization_id (str):
        names (Union[Unset, list[str]]):
        offset (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 20.
        sorts (Union[Unset, list[ListTeamsSortsItem]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Error, TeamsPaginated]]
    """

    kwargs = _get_kwargs(
        organization_id=organization_id,
        names=names,
        offset=offset,
        limit=limit,
        sorts=sorts,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    organization_id: str,
    *,
    client: AuthenticatedClient,
    names: Union[Unset, list[str]] = UNSET,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 20,
    sorts: Union[Unset, list[ListTeamsSortsItem]] = UNSET,
) -> Optional[Union[Error, TeamsPaginated]]:
    """List Teams

     List all Teams in an Organization.

    Args:
        organization_id (str):
        names (Union[Unset, list[str]]):
        offset (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 20.
        sorts (Union[Unset, list[ListTeamsSortsItem]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Error, TeamsPaginated]
    """

    return sync_detailed(
        organization_id=organization_id,
        client=client,
        names=names,
        offset=offset,
        limit=limit,
        sorts=sorts,
    ).parsed


async def asyncio_detailed(
    organization_id: str,
    *,
    client: AuthenticatedClient,
    names: Union[Unset, list[str]] = UNSET,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 20,
    sorts: Union[Unset, list[ListTeamsSortsItem]] = UNSET,
) -> Response[Union[Error, TeamsPaginated]]:
    """List Teams

     List all Teams in an Organization.

    Args:
        organization_id (str):
        names (Union[Unset, list[str]]):
        offset (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 20.
        sorts (Union[Unset, list[ListTeamsSortsItem]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Error, TeamsPaginated]]
    """

    kwargs = _get_kwargs(
        organization_id=organization_id,
        names=names,
        offset=offset,
        limit=limit,
        sorts=sorts,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    organization_id: str,
    *,
    client: AuthenticatedClient,
    names: Union[Unset, list[str]] = UNSET,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 20,
    sorts: Union[Unset, list[ListTeamsSortsItem]] = UNSET,
) -> Optional[Union[Error, TeamsPaginated]]:
    """List Teams

     List all Teams in an Organization.

    Args:
        organization_id (str):
        names (Union[Unset, list[str]]):
        offset (Union[Unset, int]):  Default: 0.
        limit (Union[Unset, int]):  Default: 20.
        sorts (Union[Unset, list[ListTeamsSortsItem]]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Error, TeamsPaginated]
    """

    return (
        await asyncio_detailed(
            organization_id=organization_id,
            client=client,
            names=names,
            offset=offset,
            limit=limit,
            sorts=sorts,
        )
    ).parsed



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.basic_subject_profile import BasicSubjectProfile


T = TypeVar("T", bound="Invite")


@_attrs_define
class Invite:
    """
    Attributes:
        expires_at (str): The time when the invite is expired in UTC, formatted as `YYYY-MM-DDTHH:MM:SSZ`. Example:
            2022-11-22T04:37:12Z.
        invite_id (str): The invite ID. Example: clm9t1g17000008jmfsw20lsz.
        invitee (BasicSubjectProfile):
        inviter (BasicSubjectProfile):
        organization_id (str): The ID of the Organization where the invite was sent. Example: clm9t0gbt000108jv4f1cfu8u.
        organization_name (Union[Unset, str]): The name of the Organization where the invite was sent. Example: My
            Organization.
        user_id (Union[Unset, str]): The ID for the user who was invited. Example: clm9t060z000008jv3mira7x5.
    """

    expires_at: str
    invite_id: str
    invitee: "BasicSubjectProfile"
    inviter: "BasicSubjectProfile"
    organization_id: str
    organization_name: Union[Unset, str] = UNSET
    user_id: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        expires_at = self.expires_at

        invite_id = self.invite_id

        invitee = self.invitee.to_dict()

        inviter = self.inviter.to_dict()

        organization_id = self.organization_id

        organization_name = self.organization_name

        user_id = self.user_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "expiresAt": expires_at,
                "inviteId": invite_id,
                "invitee": invitee,
                "inviter": inviter,
                "organizationId": organization_id,
            }
        )
        if organization_name is not UNSET:
            field_dict["organizationName"] = organization_name
        if user_id is not UNSET:
            field_dict["userId"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.basic_subject_profile import BasicSubjectProfile

        d = dict(src_dict)
        expires_at = d.pop("expiresAt")

        invite_id = d.pop("inviteId")

        invitee = BasicSubjectProfile.from_dict(d.pop("invitee"))

        inviter = BasicSubjectProfile.from_dict(d.pop("inviter"))

        organization_id = d.pop("organizationId")

        organization_name = d.pop("organizationName", UNSET)

        user_id = d.pop("userId", UNSET)

        invite = cls(
            expires_at=expires_at,
            invite_id=invite_id,
            invitee=invitee,
            inviter=inviter,
            organization_id=organization_id,
            organization_name=organization_name,
            user_id=user_id,
        )

        invite.additional_properties = d
        return invite

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.team import Team


T = TypeVar("T", bound="TeamsPaginated")


@_attrs_define
class TeamsPaginated:
    """
    Attributes:
        limit (int): The maximum number of Teams in one page. Example: 10.
        offset (int): The offset of the current page of Teams.
        teams (list['Team']): The list of Teams in the current page.
        total_count (int): The total number of Teams. Example: 100.
    """

    limit: int
    offset: int
    teams: list["Team"]
    total_count: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        limit = self.limit

        offset = self.offset

        teams = []
        for teams_item_data in self.teams:
            teams_item = teams_item_data.to_dict()
            teams.append(teams_item)

        total_count = self.total_count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "limit": limit,
                "offset": offset,
                "teams": teams,
                "totalCount": total_count,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.team import Team

        d = dict(src_dict)
        limit = d.pop("limit")

        offset = d.pop("offset")

        teams = []
        _teams = d.pop("teams")
        for teams_item_data in _teams:
            teams_item = Team.from_dict(teams_item_data)

            teams.append(teams_item)

        total_count = d.pop("totalCount")

        teams_paginated = cls(
            limit=limit,
            offset=offset,
            teams=teams,
            total_count=total_count,
        )

        teams_paginated.additional_properties = d
        return teams_paginated

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_team_request_organization_role import CreateTeamRequestOrganizationRole
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateTeamRequest")


@_attrs_define
class CreateTeamRequest:
    """
    Attributes:
        name (str): The Team's name. Example: My Team.
        description (Union[Unset, str]): The Team's description. Example: My Team description.
        member_ids (Union[Unset, list[str]]): The list of IDs for users to add to the Team. Example:
            ['clma67byh000008md1gr995ez'].
        organization_role (Union[Unset, CreateTeamRequestOrganizationRole]): The Team's Organization role. Example:
            ORGANIZATION_MEMBER.
    """

    name: str
    description: Union[Unset, str] = UNSET
    member_ids: Union[Unset, list[str]] = UNSET
    organization_role: Union[Unset, CreateTeamRequestOrganizationRole] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description = self.description

        member_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.member_ids, Unset):
            member_ids = self.member_ids

        organization_role: Union[Unset, str] = UNSET
        if not isinstance(self.organization_role, Unset):
            organization_role = self.organization_role.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if member_ids is not UNSET:
            field_dict["memberIds"] = member_ids
        if organization_role is not UNSET:
            field_dict["organizationRole"] = organization_role

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        description = d.pop("description", UNSET)

        member_ids = cast(list[str], d.pop("memberIds", UNSET))

        _organization_role = d.pop("organizationRole", UNSET)
        organization_role: Union[Unset, CreateTeamRequestOrganizationRole]
        if isinstance(_organization_role, Unset):
            organization_role = UNSET
        else:
            organization_role = CreateTeamRequestOrganizationRole(_organization_role)

        create_team_request = cls(
            name=name,
            description=description,
            member_ids=member_ids,
            organization_role=organization_role,
        )

        create_team_request.additional_properties = d
        return create_team_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListUsersSortsItem(str, Enum):
    CREATEDATASC = "createdAt:asc"
    CREATEDATDESC = "createdAt:desc"
    FULLNAMEASC = "fullName:asc"
    FULLNAMEDESC = "fullName:desc"
    IDASC = "id:asc"
    IDDESC = "id:desc"
    UPDATEDATASC = "updatedAt:asc"
    UPDATEDATDESC = "updatedAt:desc"
    USERNAMEASC = "username:asc"
    USERNAMEDESC = "username:desc"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.user import User


T = TypeVar("T", bound="UsersPaginated")


@_attrs_define
class UsersPaginated:
    """
    Attributes:
        limit (int): The maximum number of users in one page. Example: 10.
        offset (int): The offset of the current page of users.
        total_count (int): The total number of users. Example: 100.
        users (list['User']): The list of users in the current page.
    """

    limit: int
    offset: int
    total_count: int
    users: list["User"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        limit = self.limit

        offset = self.offset

        total_count = self.total_count

        users = []
        for users_item_data in self.users:
            users_item = users_item_data.to_dict()
            users.append(users_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "limit": limit,
                "offset": offset,
                "totalCount": total_count,
                "users": users,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user import User

        d = dict(src_dict)
        limit = d.pop("limit")

        offset = d.pop("offset")

        total_count = d.pop("totalCount")

        users = []
        _users = d.pop("users")
        for users_item_data in _users:
            users_item = User.from_dict(users_item_data)

            users.append(users_item)

        users_paginated = cls(
            limit=limit,
            offset=offset,
            total_count=total_count,
            users=users,
        )

        users_paginated.additional_properties = d
        return users_paginated

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_api_token_request_type import CreateApiTokenRequestType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateApiTokenRequest")


@_attrs_define
class CreateApiTokenRequest:
    """
    Attributes:
        name (str): The name of the API token. Example: My token.
        role (str): The role of the API token. Example: WORKSPACE_OWNER.
        type_ (CreateApiTokenRequestType): The scope of the API token. Example: WORKSPACE.
        description (Union[Unset, str]): The description for the API token. Example: This is my API token.
        entity_id (Union[Unset, str]): The ID of the Workspace or Deployment to which the API token is scoped. It is
            required if `Type` is `WORKSPACE` or `DEPLOYMENT`. Example: clm8pxjjw000008l23jm08hyu.
        token_expiry_period_in_days (Union[Unset, int]): The expiry period of the API token in days. If not specified,
            the token will never expire. Example: 30.
    """

    name: str
    role: str
    type_: CreateApiTokenRequestType
    description: Union[Unset, str] = UNSET
    entity_id: Union[Unset, str] = UNSET
    token_expiry_period_in_days: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        role = self.role

        type_ = self.type_.value

        description = self.description

        entity_id = self.entity_id

        token_expiry_period_in_days = self.token_expiry_period_in_days

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "role": role,
                "type": type_,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if entity_id is not UNSET:
            field_dict["entityId"] = entity_id
        if token_expiry_period_in_days is not UNSET:
            field_dict["tokenExpiryPeriodInDays"] = token_expiry_period_in_days

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        role = d.pop("role")

        type_ = CreateApiTokenRequestType(d.pop("type"))

        description = d.pop("description", UNSET)

        entity_id = d.pop("entityId", UNSET)

        token_expiry_period_in_days = d.pop("tokenExpiryPeriodInDays", UNSET)

        create_api_token_request = cls(
            name=name,
            role=role,
            type_=type_,
            description=description,
            entity_id=entity_id,
            token_expiry_period_in_days=token_expiry_period_in_days,
        )

        create_api_token_request.additional_properties = d
        return create_api_token_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.role_scope_type import RoleScopeType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.basic_subject_profile import BasicSubjectProfile


T = TypeVar("T", bound="Role")


@_attrs_define
class Role:
    """
    Attributes:
        created_at (str): The time the role was created.
        created_by (BasicSubjectProfile):
        id (str): The role's ID. Example: cluc9tapx000901qn2xrgqdmn.
        name (str): The role's name. Example: Deployment_Viewer.
        restricted_workspace_ids (list[str]): The IDs of Workspaces that the role is restricted to. Example:
            ['cldbvzoi20182g8odxt8ehi5i'].
        scope_type (RoleScopeType): The role's scope. Example: DEPLOYMENT.
        updated_at (str): The time the role was last updated.
        updated_by (BasicSubjectProfile):
        description (Union[Unset, str]): The role's description. Example: Subject can only view deployments..
    """

    created_at: str
    created_by: "BasicSubjectProfile"
    id: str
    name: str
    restricted_workspace_ids: list[str]
    scope_type: RoleScopeType
    updated_at: str
    updated_by: "BasicSubjectProfile"
    description: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at

        created_by = self.created_by.to_dict()

        id = self.id

        name = self.name

        restricted_workspace_ids = self.restricted_workspace_ids

        scope_type = self.scope_type.value

        updated_at = self.updated_at

        updated_by = self.updated_by.to_dict()

        description = self.description

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "createdBy": created_by,
                "id": id,
                "name": name,
                "restrictedWorkspaceIds": restricted_workspace_ids,
                "scopeType": scope_type,
                "updatedAt": updated_at,
                "updatedBy": updated_by,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.basic_subject_profile import BasicSubjectProfile

        d = dict(src_dict)
        created_at = d.pop("createdAt")

        created_by = BasicSubjectProfile.from_dict(d.pop("createdBy"))

        id = d.pop("id")

        name = d.pop("name")

        restricted_workspace_ids = cast(list[str], d.pop("restrictedWorkspaceIds"))

        scope_type = RoleScopeType(d.pop("scopeType"))

        updated_at = d.pop("updatedAt")

        updated_by = BasicSubjectProfile.from_dict(d.pop("updatedBy"))

        description = d.pop("description", UNSET)

        role = cls(
            created_at=created_at,
            created_by=created_by,
            id=id,
            name=name,
            restricted_workspace_ids=restricted_workspace_ids,
            scope_type=scope_type,
            updated_at=updated_at,
            updated_by=updated_by,
            description=description,
        )

        role.additional_properties = d
        return role

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.permission_entry import PermissionEntry


T = TypeVar("T", bound="PermissionGroup")


@_attrs_define
class PermissionGroup:
    """
    Attributes:
        description (str): The permission group's description. Example: Astro notification channel defines where alert
            messages can be sent. For example, alert messages issued via email or slack..
        name (str): The permission group's name. Example: workspace.notificationChannels.
        permissions (list['PermissionEntry']): The permission group's permissions.
        scope (str): The permission group's scope. Example: Workspace NotificationChannels.
    """

    description: str
    name: str
    permissions: list["PermissionEntry"]
    scope: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        description = self.description

        name = self.name

        permissions = []
        for permissions_item_data in self.permissions:
            permissions_item = permissions_item_data.to_dict()
            permissions.append(permissions_item)

        scope = self.scope

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "description": description,
                "name": name,
                "permissions": permissions,
                "scope": scope,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.permission_entry import PermissionEntry

        d = dict(src_dict)
        description = d.pop("description")

        name = d.pop("name")

        permissions = []
        _permissions = d.pop("permissions")
        for permissions_item_data in _permissions:
            permissions_item = PermissionEntry.from_dict(permissions_item_data)

            permissions.append(permissions_item)

        scope = d.pop("scope")

        permission_group = cls(
            description=description,
            name=name,
            permissions=permissions,
            scope=scope,
        )

        permission_group.additional_properties = d
        return permission_group

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.api_token_role_entity_type import ApiTokenRoleEntityType

T = TypeVar("T", bound="ApiTokenRole")


@_attrs_define
class ApiTokenRole:
    """
    Attributes:
        entity_id (str): The ID of the entity to which the API token is scoped for. For example, for Workspace API
            tokens, this is the Workspace ID. Example: clm8sgvai000008l794psbkdv.
        entity_type (ApiTokenRoleEntityType): The type of the entity to which the API token is scoped for. Example:
            WORKSPACE.
        role (str): The role of the API token. Example: WORKSPACE_MEMBER.
    """

    entity_id: str
    entity_type: ApiTokenRoleEntityType
    role: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        entity_id = self.entity_id

        entity_type = self.entity_type.value

        role = self.role

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "entityId": entity_id,
                "entityType": entity_type,
                "role": role,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        entity_id = d.pop("entityId")

        entity_type = ApiTokenRoleEntityType(d.pop("entityType"))

        role = d.pop("role")

        api_token_role = cls(
            entity_id=entity_id,
            entity_type=entity_type,
            role=role,
        )

        api_token_role.additional_properties = d
        return api_token_role

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
